"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
const datastore_1 = require("@aws-amplify/datastore");
const react_hooks_1 = require("@testing-library/react-hooks");
const useDataStoreBinding_1 = require("../useDataStoreBinding");
const models_1 = require("./test-models/models");
const createDataStorePredicate_1 = require("../createDataStorePredicate");
jest.mock('@aws-amplify/datastore');
const fakeModel = {
    id: 'FakeModel',
};
const nextFakeModel = {
    id: 'nextFakeModel',
};
const fakeItem = {
    fakeField: 'fake-value',
};
const fakePagination = {
    limit: 100,
    sort: (s) => s.rating(datastore_1.SortDirection.ASCENDING),
};
const fakeId = 'fake-id';
describe('useDataStoreCollection', () => {
    afterEach(() => jest.clearAllMocks());
    it('should return default values while data is being fetched', () => {
        datastore_1.DataStore.observeQuery.mockImplementation(() => ({
            subscribe: () => ({
                unsubscribe: jest.fn(),
            }),
        }));
        const { result } = (0, react_hooks_1.renderHook)(() => (0, useDataStoreBinding_1.useDataStoreCollection)({
            model: fakeModel,
        }));
        expect(result.current.isLoading).toBe(true);
        expect(result.current.items).toHaveLength(0);
        expect(result.current.error).toBeUndefined();
    });
    it('should set error if DataStore.observeQuery throws an error', async () => {
        const fakeError = new Error('Unexpected DataStore error');
        const fakeDataStoreObserveQuery = jest.fn(() => ({
            // eslint-disable-next-line @typescript-eslint/no-unused-vars
            subscribe: (_, onError) => {
                setTimeout(() => onError(fakeError), 500);
                return { unsubscribe: () => { } };
            },
        }));
        datastore_1.DataStore.observeQuery.mockImplementation(fakeDataStoreObserveQuery);
        const { result, waitForNextUpdate } = (0, react_hooks_1.renderHook)(() => (0, useDataStoreBinding_1.useDataStoreCollection)({
            model: fakeModel,
        }));
        // Trigger fetch
        await waitForNextUpdate();
        // Check if error is set and loading state is back to normal
        expect(result.current.error).toBe(fakeError);
        expect(result.current.isLoading).toBe(false);
        // Finally, check returned items
        expect(result.current.items).toHaveLength(0);
    });
    it('should return items on success', async () => {
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const fakeItems = Array.from({ length: 100 }).map((_) => fakeItem);
        const namePredicateObject = {
            field: 'name',
            operator: 'eq',
            operand: 'fake-value',
        };
        const predicate = (0, createDataStorePredicate_1.createDataStorePredicate)(namePredicateObject);
        const fakeDataStoreObserveQuery = jest.fn(() => ({
            subscribe: (onSuccess) => {
                setTimeout(() => onSuccess({ items: fakeItems }), 500);
                return { unsubscribe: () => { } };
            },
        }));
        datastore_1.DataStore.observeQuery.mockImplementation(fakeDataStoreObserveQuery);
        const { result, waitForNextUpdate } = (0, react_hooks_1.renderHook)(() => (0, useDataStoreBinding_1.useDataStoreCollection)({
            model: fakeModel,
            criteria: predicate,
            pagination: fakePagination,
        }));
        await waitForNextUpdate();
        expect(result.current.isLoading).toBe(false);
        expect(result.current.error).toBeUndefined();
        expect(result.current.items).toBe(fakeItems);
    });
    it('should unsubscribe on unmount', () => {
        const unsubscribe = jest.fn();
        const fakeDataStoreObserveQuery = jest.fn(() => ({
            subscribe: () => ({ unsubscribe }),
        }));
        datastore_1.DataStore.observeQuery.mockImplementation(fakeDataStoreObserveQuery);
        const { unmount } = (0, react_hooks_1.renderHook)(() => (0, useDataStoreBinding_1.useDataStoreCollection)({
            model: fakeModel,
        }));
        // Force component unmount
        unmount();
        expect(unsubscribe).toHaveBeenCalled();
    });
    it('should only call DataStore.observeQuery once', () => {
        const unsubscribe = jest.fn();
        const fakeDataStoreObserveQuery = jest.fn(() => ({
            subscribe: () => ({ unsubscribe }),
        }));
        datastore_1.DataStore.observeQuery.mockImplementation(fakeDataStoreObserveQuery);
        const { rerender } = (0, react_hooks_1.renderHook)((params) => (0, useDataStoreBinding_1.useDataStoreCollection)(params), {
            initialProps: {
                model: fakeModel,
            },
        });
        expect(datastore_1.DataStore.observeQuery).toHaveBeenCalledTimes(1);
        rerender({ model: nextFakeModel });
        expect(datastore_1.DataStore.observeQuery).toHaveBeenCalledTimes(1);
    });
});
describe('useDataStoreItem', () => {
    afterEach(() => jest.clearAllMocks());
    it('should return default values while data is being fetched', async () => {
        datastore_1.DataStore.query.mockResolvedValue(undefined);
        const { result, waitForNextUpdate } = (0, react_hooks_1.renderHook)(() => (0, useDataStoreBinding_1.useDataStoreItem)({
            model: fakeModel,
            id: fakeId,
        }));
        expect(result.current.isLoading).toBe(true);
        expect(result.current.item).toBeUndefined();
        expect(result.current.error).toBeUndefined();
        await waitForNextUpdate();
    });
    it('should set error if DataStore.query throws an error', async () => {
        const fakeError = new Error('Unexpected DataStore error');
        const fakeDataStoreQuery = jest.fn(() => Promise.reject(fakeError));
        datastore_1.DataStore.query.mockImplementation(fakeDataStoreQuery);
        const { result, waitForNextUpdate } = (0, react_hooks_1.renderHook)(() => (0, useDataStoreBinding_1.useDataStoreItem)({
            model: fakeModel,
            id: fakeId,
        }));
        // Trigger fetch
        await waitForNextUpdate();
        // Check if error is set and loading state is back to normal
        expect(result.current.error).toBe(fakeError);
        expect(result.current.isLoading).toBe(false);
        // Finally, check returned item
        expect(result.current.item).toBeUndefined();
    });
    it('should return item on success', async () => {
        const fakeDataStoreQuery = jest.fn(() => Promise.resolve(fakeItem));
        datastore_1.DataStore.query.mockImplementation(fakeDataStoreQuery);
        const { result, waitForNextUpdate } = (0, react_hooks_1.renderHook)(() => (0, useDataStoreBinding_1.useDataStoreItem)({
            model: fakeModel,
            id: fakeId,
        }));
        // Check if DataStore.query was invoked with expected parameters
        expect(fakeDataStoreQuery).toHaveBeenCalledWith(fakeModel, fakeId);
        // Trigger fetch
        await waitForNextUpdate();
        // Check if there's no errors and loading state is back to normal
        expect(result.current.error).toBeUndefined();
        expect(result.current.isLoading).toBe(false);
        // Finally, check returned item
        expect(result.current.item).toBe(fakeItem);
    });
    it('should only call DataStore.query once', async () => {
        const fakeDataStoreQuery = jest.fn(() => Promise.resolve(fakeItem));
        datastore_1.DataStore.query.mockImplementation(fakeDataStoreQuery);
        const { rerender, waitForNextUpdate } = (0, react_hooks_1.renderHook)((props) => (0, useDataStoreBinding_1.useDataStoreItem)(props), {
            initialProps: {
                model: fakeModel,
                id: fakeId,
            },
        });
        // await fetch
        await waitForNextUpdate();
        expect(datastore_1.DataStore.query).toHaveBeenCalledTimes(1);
        rerender({ model: nextFakeModel, id: fakeId });
        expect(datastore_1.DataStore.query).toHaveBeenCalledTimes(1);
    });
});
describe('useDataStoreBinding', () => {
    afterEach(() => jest.clearAllMocks());
    it('handles calls with type record in the happy path', async () => {
        const fakeDataStoreQuery = jest.fn(() => Promise.resolve(fakeItem));
        datastore_1.DataStore.query.mockImplementation(fakeDataStoreQuery);
        const { result, waitForNextUpdate } = (0, react_hooks_1.renderHook)(() => (0, useDataStoreBinding_1.useDataStoreBinding)({
            model: fakeModel,
            id: fakeId,
            type: 'record',
        }));
        // Check if DataStore.query was invoked with expected parameters
        expect(fakeDataStoreQuery).toHaveBeenCalledWith(fakeModel, fakeId);
        // Trigger fetch
        await waitForNextUpdate();
        // Check if there's no errors and loading state is back to normal
        expect(result.current.error).toBeUndefined();
        expect(result.current.isLoading).toBe(false);
        // Finally, check returned item
        expect(result.current.item).toBe(fakeItem);
    });
    it('handles calls with type collection in the happy path', async () => {
        // eslint-disable-next-line @typescript-eslint/no-unused-vars
        const fakeItems = Array.from({ length: 100 }).map((_) => fakeItem);
        const namePredicateObject = {
            field: 'name',
            operator: 'eq',
            operand: 'fake-value',
        };
        const predicate = (0, createDataStorePredicate_1.createDataStorePredicate)(namePredicateObject);
        const fakeDataStoreObserveQuery = jest.fn(() => ({
            subscribe: (onSuccess) => {
                setTimeout(() => onSuccess({ items: fakeItems }), 500);
                return { unsubscribe: () => { } };
            },
        }));
        datastore_1.DataStore.observeQuery.mockImplementation(fakeDataStoreObserveQuery);
        const { result, waitForNextUpdate } = (0, react_hooks_1.renderHook)(() => (0, useDataStoreBinding_1.useDataStoreBinding)({
            model: models_1.Todo,
            criteria: predicate,
            pagination: fakePagination,
            type: 'collection',
        }));
        await waitForNextUpdate();
        expect(result.current.isLoading).toBe(false);
        expect(result.current.error).toBeUndefined();
        expect(result.current.items).toBe(fakeItems);
    });
});
//# sourceMappingURL=useDataStore.test.js.map